# -*- coding: utf-8 -*-

"""
Uses Wannier converter to write the results to the h5 archive.
"""

import os
import sys
import numpy as np

from h5 import HDFArchive

def read_u_file(file):
    for line in file:
        if line.count('.') == 2:
            yield line

def read_files(seeds):
    proj_mats = {}

    for seed in seeds:
        print(seed, flush=True)
        # Reads U matrix
        with open(f'{seed}_u.mat', 'r') as file:
            file.readline()
            n_k, _, _ = (int(s) for s in file.readline().split())
            u_mat = np.loadtxt(read_u_file(file))

        n_wf = int(np.sqrt(np.prod(u_mat.shape)//2//n_k) + .5)
        u_mat = (u_mat[:, 0] + 1j*u_mat[:, 1]).reshape(n_k, n_wf, n_wf)

        # Reads U_dis matrix if existent
        u_dis_exists = os.path.exists(f'{seed}_u_dis.mat')
        if u_dis_exists:
            with open(f'{seed}_u_dis.mat', 'r') as file:
                u_dis_mat = np.loadtxt(read_u_file(file))
            u_dis_mat = (u_dis_mat[:, 0] + 1j*u_dis_mat[:, 1]).reshape(n_k, n_wf, -1)
        else:
            u_dis_mat = np.array([np.eye(n_wf) for _ in range(n_k)])

        # Creates proj matrix
        proj_mat = np.einsum('kcb,kdc->kdb', u_dis_mat, u_mat).conj()
        proj_mats[seed] = proj_mat

        # Checks unity of proj matrix
        unity = np.einsum('kab,kcb->kac', proj_mat, proj_mat.conj())
        for u in unity:
            assert np.allclose(np.eye(n_wf), u)

    return proj_mats

def write_to_h5(file_name, proj_mats):
    with HDFArchive(file_name, 'a') as archive:
        if 'dft_alt_projectors' not in archive:
            archive.create_group('dft_alt_projectors')

        for name, proj_mat in proj_mats.items():
            archive['dft_alt_projectors'][name] = proj_mat

if __name__ == '__main__':
    assert len(sys.argv) > 2, 'Provide h5 archive and seeds'
    proj_mats = read_files(sys.argv[2:])
    write_to_h5(sys.argv[1], proj_mats)

